@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 Google LLC.
+# Copyright 2022 Google LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,7 +23,7 @@ import numpy as np
 
 
 def fixpoint_iter(cond_fn: Callable[[int, Any, Any], bool],
-                  body_fn: Callable[[Any], Any],
+                  body_fn: Callable[[Any, Any, Any, Any], Any],
                   min_iterations: int,
                   max_iterations: int,
                   inner_iterations: int,
