@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 Google LLC.
+# Copyright 2022 Google LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@
 # Lint as: python3
 """A class describing operations used to instantiate and use a geometry."""
 import functools
-from typing import Optional, Sequence, Union
+from typing import Optional, Union
 
 import jax
 import jax.numpy as jnp
@@ -51,7 +51,8 @@ class Geometry:
                kernel_matrix: Optional[jnp.ndarray] = None,
                epsilon: Union[epsilon_scheduler.Epsilon, float, None] = None,
                relative_epsilon: Optional[bool] = None,
-               scale: float = None,
+               scale_epsilon: Optional[float] = None,
+               scale_cost: Optional[Union[float, str]] = None,
                **kwargs):
     r"""Initializes a geometry by passing it a cost matrix or a kernel matrix.
 
@@ -68,32 +69,40 @@ class Geometry:
         the mean value of the ``cost_matrix``.
       relative_epsilon: whether epsilon is passed relative to scale of problem,
         here understood as mean value of ``cost_matrix``.
-      scale: the scale multiplier for epsilon.
+      scale_epsilon: the scale multiplier for epsilon.
+      scale_cost: option to rescale the cost matrix. Implemented scalings are
+        'median', 'mean' and 'max_cost'. Alternatively, a float factor can be
+        given to rescale the cost such that ``cost_matrix /= scale_cost``.
       **kwargs: additional kwargs to epsilon.
     """
     self._cost_matrix = cost_matrix
     self._kernel_matrix = kernel_matrix
     self._epsilon_init = epsilon
     self._relative_epsilon = relative_epsilon
-    self._scale = scale
+    self._scale_epsilon = scale_epsilon
+    self._scale_cost = scale_cost
     # Define default dictionary and update it with user's values.
     self._kwargs = {**{'init': None, 'decay': None}, **kwargs}
 
   @property
-  def scale(self) -> float:
+  def cost_rank(self):
+    return None
+
+  @property
+  def scale_epsilon(self) -> float:
     """Computes the scale of the epsilon, potentially based on data."""
     if isinstance(self._epsilon_init, epsilon_scheduler.Epsilon):
       return 1.0
 
     rel = self._relative_epsilon
-    trigger = ((self._scale is None) and
+    trigger = ((self._scale_epsilon is None) and
                (rel or rel is None) and
                (self._epsilon_init is None or rel))
-    if (self._scale is None) and (trigger is not None):  # for dry run
+    if (self._scale_epsilon is None) and (trigger is not None):  # for dry run
       return jnp.where(
           trigger, jax.lax.stop_gradient(self.mean_cost_matrix), 1.0)
     else:
-      return self._scale
+      return self._scale_epsilon
 
   @property
   def _epsilon(self):
@@ -101,13 +110,19 @@ class Geometry:
     if isinstance(self._epsilon_init, epsilon_scheduler.Epsilon):
       return self._epsilon_init
     eps = 5e-2 if self._epsilon_init is None else self._epsilon_init
-    return epsilon_scheduler.Epsilon.make(eps, scale=self.scale, **self._kwargs)
+    return epsilon_scheduler.Epsilon.make(
+        eps, scale_epsilon=self.scale_epsilon, **self._kwargs)
 
   @property
   def cost_matrix(self):
+    """Returns cost matrix, computes it if only kernel was specified."""
     if self._cost_matrix is None:
-      return -self.epsilon * jnp.log(self._kernel_matrix)
-    return self._cost_matrix
+      # If no epsilon was passed on to the geometry, then assume it is one by
+      # default.
+      cost = -jnp.log(self._kernel_matrix)
+      cost *= self.scale_cost
+      return cost if self._epsilon_init is None else self.epsilon * cost
+    return self._cost_matrix * self.scale_cost
 
   @property
   def median_cost_matrix(self):
@@ -124,7 +139,8 @@ class Geometry:
   @property
   def kernel_matrix(self):
     if self._kernel_matrix is None:
-      return jnp.exp(-(self._cost_matrix / self.epsilon))
+      return jnp.exp(
+          -(self._cost_matrix / self.epsilon))**(1.0 / self.scale_cost)
     return self._kernel_matrix
 
   @property
@@ -133,25 +149,50 @@ class Geometry:
 
   @property
   def shape(self):
-    mat = self.kernel_matrix if self.cost_matrix is None else self.cost_matrix
+    mat = (self._kernel_matrix if self._cost_matrix is None
+           else self._cost_matrix)
     if mat is not None:
       return mat.shape
     return (0, 0)
 
+  @property
+  def is_squared_euclidean(self):
+    return False
+
+  @property
+  def is_online(self):
+    return False
+
   @property
   def is_symmetric(self):
     mat = self.kernel_matrix if self.cost_matrix is None else self.cost_matrix
     return (mat.shape[0] == mat.shape[1] and
             jnp.all(mat == mat.T)) if mat is not None else False
 
+  @property
+  def scale_cost(self):
+    """Computes the factor to scale the cost matrix."""
+    if isinstance(self._scale_cost, float):
+      return 1.0 / self._scale_cost
+    elif self._scale_cost == 'max_cost':
+      return jax.lax.stop_gradient(1.0 / jnp.max(self._cost_matrix))
+    elif self._scale_cost == 'mean':
+      return jax.lax.stop_gradient(1.0 / jnp.mean(self._cost_matrix))
+    elif self._scale_cost == 'median':
+      return jax.lax.stop_gradient(1.0 / jnp.median(self._cost_matrix))
+    elif isinstance(self._scale_cost, str):
+      raise ValueError(f'Scaling {self._scale_cost} not implemented.')
+    else:
+      return 1.0
+
   def copy_epsilon(self, other):
     """Copies the epsilon parameters from another geometry."""
     scheduler = other._epsilon
     self._epsilon_init = scheduler._target_init
     self._relative_epsilon = False
-    self._scale = other.scale
+    self._scale_epsilon = other.scale_epsilon
 
-  # The functions below are at the core of Sinkhorn iterations, they are
+  # The functions below are at the core of Sinkhorn iterations, they
   # are implemented here in their default form, either in lse (using directly
   # cost matrices in stabilized form) or kernel mode (using kernel matrices).
 
@@ -257,35 +298,6 @@ class Geometry:
 
   # Functions that are not supposed to be changed by inherited classes.
   # These are the point of entry for Sinkhorn's algorithm to use a geometry.
-  def error(self,
-            f_u: jnp.ndarray,
-            g_v: jnp.ndarray,
-            target: jnp.ndarray,
-            axis: int = 0,
-            norm_error: Sequence[int] = (1,),
-            lse_mode: bool = True):
-    """Outputs error, using 2 potentials/scalings, of transport w.r.t target marginal.
-
-    Args:
-      f_u: a vector of potentials or scalings for the first marginal.
-      g_v: a vector of potentials or scalings for the second marginal.
-      target: target marginal.
-      axis: axis (0 or 1) along which to compute marginal.
-      norm_error: (t-uple of int) p's to compute p-norm between marginal/target
-      lse_mode: whether operating on scalings or potentials
-
-    Returns:
-      t-uple of floats, quantifying difference between target / marginal.
-    """
-    if lse_mode:
-      marginal = self.marginal_from_potentials(f_u, g_v, axis=axis)
-    else:
-      marginal = self.marginal_from_scalings(f_u, g_v, axis=axis)
-    norm_error = jnp.array(norm_error)
-    error = jnp.sum(
-        jnp.abs(marginal - target)**norm_error[:, jnp.newaxis],
-        axis=1)**(1.0 / norm_error)
-    return error
 
   def update_potential(self, f, g, log_marginal, iteration=None, axis=0):
     """Carries out one Sinkhorn update for potentials, i.e.
@@ -418,6 +430,11 @@ class Geometry:
                      jnp.exp(jnp.where(finite, potential / self.epsilon, 0.0)),
                      0.0)
 
+  def apply_square_cost(self, arr: jnp.ndarray, axis: int = 0) -> jnp.ndarray:
+    """Applies elementwise-square of cost matrix to array (vector or matrix)."""
+    fn = lambda x: x ** 2
+    return self.apply_cost(arr, axis, fn)
+
   def apply_cost(self, arr: jnp.ndarray, axis: int = 0, fn=None) -> jnp.ndarray:
     """Applies cost matrix to array (vector or matrix).
 
@@ -429,7 +446,7 @@ class Geometry:
     Args:
       arr: jnp.ndarray [num_a or num_b, p], vector that will be multiplied by
         the cost matrix.
-      axis: standard cost matrix if axis=1, transport if 0
+      axis: standard cost matrix if axis=1, transpose if 0
       fn: function to apply to cost matrix element-wise before the dot product
 
     Returns:
@@ -449,6 +466,18 @@ class Geometry:
     )(
         arr)
 
+  def rescale_cost_fn(self, factor: float):
+    if self._cost_matrix is not None:
+      self._cost_matrix *= factor
+    if self._kernel_matrix is not None:
+      self._kernel_matrix **= 1/factor
+
+  def apply_cost_1(self, vec, axis=0):
+    pass
+
+  def apply_cost_2(self, vec, axis=0):
+    pass
+
   def _apply_cost_to_vec(self,
                          vec: jnp.ndarray,
                          axis: int = 0,
@@ -482,9 +511,21 @@ class Geometry:
 
   def tree_flatten(self):
     return (self._cost_matrix, self._kernel_matrix, self._epsilon_init,
-            self._relative_epsilon, self._kwargs), None
+            self._relative_epsilon,
+            self._kwargs), {'scale_cost': self._scale_cost}
 
   @classmethod
   def tree_unflatten(cls, aux_data, children):
-    del aux_data
-    return cls(*children[:-1], **children[-1])
+    return cls(*children[:-1], **children[-1], **aux_data)
+
+
+def is_affine(fn) -> bool:
+  """Tests heuristically if a function is affine."""
+  x = jnp.arange(10.0)
+  out = jax.vmap(jax.grad(fn))(x)
+  return jnp.sum(jnp.diff(jnp.abs(out))) == 0.0
+
+
+def is_linear(fn) -> bool:
+  """Tests heuristically if a function is linear."""
+  return fn(0.0) == 0.0 and is_affine(fn)
