@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 Google LLC.
+# Copyright 2022 Google LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,8 @@
 
 """OTT tools: A set of tools to use OT in differentiable ML pipelines."""
 
-from . import plot
+#from . import plot
+from . import gaussian_mixture
 from . import sinkhorn_divergence
 from . import soft_sort
 from . import transport
