@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 Google LLC.
+# Copyright 2022 Google LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,11 +16,12 @@
 """Implements the sinkhorn divergence."""
 
 import collections
-from typing import Any, Dict, Optional, Type
+from typing import Any, Dict, Mapping, Optional, Type
 
 import jax
 from jax import numpy as jnp
 from ott.core import sinkhorn
+from ott.core import segment
 from ott.geometry import geometry
 from ott.geometry import pointcloud
 
@@ -65,12 +66,13 @@ def sinkhorn_divergence(
     tuple: (sinkhorn divergence value, three pairs of potentials, three costs)
   """
   geometries = geom.prepare_divergences(*args, static_b=static_b, **kwargs)
+  num_a, num_b = geometries[0].shape
+
   geometries = (geometries + (None,) * max(0, 3 - len(geometries)))[:3]
   if share_epsilon:
     for geom in filter(None, geometries[1:(2 if static_b else 3)]):
       geom.copy_epsilon(geometries[0])
 
-  num_a, num_b = geometries[0].shape
   a = jnp.ones((num_a,)) / num_a if a is None else a
   b = jnp.ones((num_b,)) / num_b if b is None else b
   div_kwargs = {} if sinkhorn_kwargs is None else sinkhorn_kwargs
@@ -111,24 +113,21 @@ def _sinkhorn_divergence(
   # on a simpler choice (parallel_dual_updates + momentum 0.5) that is known
   # to work well in such settings. In the future we might want to give some
   # freedom on setting parameters for the (x,x)/(y,y) part.
+  # Since symmetric terms are computed assuming a = b, the linear systems
+  # arising in implicit differentiation (if used) of the potentials computed for
+  # the symmetric parts should be marked as symmetric.
   kwargs_symmetric = kwargs.copy()
   kwargs_symmetric.update(
       parallel_dual_updates=True,
       momentum=0.5,
       chg_momentum_from=0,
-      anderson_acceleration=0)
-
-  # Since symmetric terms are computed assuming a = b, the linear systems
-  # arising in implicit differentiation (if used) of the potentials computed for
-  # the symmetric parts should be marked as symmetric.
-  linear_solve_kwargs = kwargs_symmetric.pop('linear_solve_kwargs', {})
-  linear_solve_kwargs.update(symmetric=True)
-  kwargs_symmetric.update(linear_solve_kwargs=linear_solve_kwargs)
+      anderson_acceleration=0,
+      implicit_solver_symmetric=True)
 
   out_xy = sinkhorn.sinkhorn(geometry_xy, a, b, **kwargs)
   out_xx = sinkhorn.sinkhorn(geometry_xx, a, a, **kwargs_symmetric)
   if geometry_yy is None:
-    out_yy = sinkhorn.SinkhornOutput(None, None, 0, None, None)
+    out_yy = sinkhorn.SinkhornOutput(errors=jnp.array([]), reg_ot_cost=0)
   else:
     out_yy = sinkhorn.sinkhorn(geometry_yy, b, b, **kwargs_symmetric)
 
@@ -141,33 +140,29 @@ def _sinkhorn_divergence(
                                   tuple(s.converged for s in out))
 
 
-def segment_sinkhorn_divergence(x: jnp.ndarray,
-                                y: jnp.ndarray,
-                                segment_ids_x: Optional[jnp.ndarray] = None,
-                                segment_ids_y: Optional[jnp.ndarray] = None,
-                                num_segments: Optional[int] = None,
-                                indices_are_sorted: Optional[bool] = None,
-                                num_per_segment_x: Optional[jnp.ndarray] = None,
-                                num_per_segment_y: Optional[jnp.ndarray] = None,
-                                weights_x: Optional[jnp.ndarray] = None,
-                                weights_y: Optional[jnp.ndarray] = None,
-                                sinkhorn_kwargs: Optional[Dict[str,
-                                                               Any]] = None,
-                                static_b: bool = False,
-                                share_epsilon: bool = True,
-                                **kwargs) -> jnp.ndarray:
+def segment_sinkhorn_divergence(
+    x: jnp.ndarray,
+    y: jnp.ndarray,
+    segment_ids_x: Optional[jnp.ndarray] = None,
+    segment_ids_y: Optional[jnp.ndarray] = None,
+    num_segments: Optional[int] = None,
+    indices_are_sorted: Optional[bool] = None,
+    num_per_segment_x: Optional[jnp.ndarray] = None,
+    num_per_segment_y: Optional[jnp.ndarray] = None,
+    weights_x: Optional[jnp.ndarray] = None,
+    weights_y: Optional[jnp.ndarray] = None,
+    sinkhorn_kwargs: Optional[Mapping[str, Any]] = None,
+    static_b: bool = False,
+    share_epsilon: bool = True,
+    **kwargs) -> jnp.ndarray:
   """Computes Sinkhorn divergence between subsets of data with pointcloud.
-
-  There are two interfaces: either use `segment_ids_x`, `segment_ids_y`, and
-  optionally `num_segments` and `indices_are_sorted`, OR use `num_per_segment_x`
-  and `num_per_segment_y`. If using the first interface, `num_segments` is
-  required for JIT compilation. Assumes range(0, `num_segments`) are the segment
-  ids. The second interface assumes `x` and `y` are segmented contiguously.
+  
+  The second interface assumes `x` and `y` are segmented contiguously.
 
   In all cases, both `x` and `y` should contain the same number of segments.
   Each segment will be separately run through the sinkhorn divergence using
   array padding.
-
+  
   Args:
     x: Array of input points, of shape [num_x, feature]. Multiple segments are
       held in this single array.
@@ -205,61 +200,24 @@ def segment_sinkhorn_divergence(x: jnp.ndarray,
   Returns:
     An array of sinkhorn divergence values for each segment.
   """
-
   use_segment_ids = segment_ids_x is not None
-
   if use_segment_ids:
     assert segment_ids_y is not None
   else:
     assert num_per_segment_x is not None
     assert num_per_segment_y is not None
 
-  if use_segment_ids:
-    if num_segments is None:
-      num_segments = jnp.max(segment_ids_x) + 1
-      assert num_segments == jnp.max(segment_ids_y) + 1
-
-    if indices_are_sorted is None:
-      indices_are_sorted = False
-
-    num_per_segment_x = jax.ops.segment_sum(
-        jnp.ones_like(segment_ids_x),
-        segment_ids_x,
-        num_segments=num_segments,
-        indices_are_sorted=indices_are_sorted)
-    num_per_segment_y = jax.ops.segment_sum(
-        jnp.ones_like(segment_ids_y),
-        segment_ids_y,
-        num_segments=num_segments,
-        indices_are_sorted=indices_are_sorted)
-  else:
-    assert num_segments is None
-    num_segments = num_per_segment_x.shape[0]
-    segment_ids_x = jnp.arange(num_segments).repeat(num_per_segment_x)
-    segment_ids_y = jnp.arange(num_segments).repeat(num_per_segment_y)
-
-  num_x = x.shape[0]
-  num_y = y.shape[0]
-
-  if weights_x is None:
-    weights_x = (1 / num_per_segment_x).repeat(
-        num_per_segment_x, total_repeat_length=num_x)
-
-  if weights_y is None:
-    weights_y = (1 / num_per_segment_y).repeat(
-        num_per_segment_y, total_repeat_length=num_y)
-
-  segmented_x = jnp.stack(
-      [x * (segment_ids_x == i)[:, None] for i in range(num_segments)])
-
-  segmented_y = jnp.stack(
-      [y * (segment_ids_y == i)[:, None] for i in range(num_segments)])
-
-  segmented_weights_x = jnp.stack(
-      [weights_x * (segment_ids_x == i) for i in range(num_segments)])
-
-  segmented_weights_y = jnp.stack(
-      [weights_y * (segment_ids_y == i) for i in range(num_segments)])
+  segmented_x, segmented_weights_x, num_segments_x = segment.segment_point_cloud(
+    x, weights_x,
+    segment_ids_x, num_segments, indices_are_sorted,
+    num_per_segment_x)
+  
+  segmented_y, segmented_weights_y, num_segments_y = segment.segment_point_cloud(
+    y, weights_y,
+    segment_ids_y, num_segments, indices_are_sorted,
+    num_per_segment_y)
+  
+  assert num_segments_x == num_segments_y
 
   def single_segment_sink_div(padded_x, padded_y, padded_weight_x,
                               padded_weight_y):
