@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 Google LLC.
+# Copyright 2022 Google LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,7 +28,7 @@ def transport_for_sort(
     inputs: jnp.ndarray,
     weights: jnp.ndarray,
     target_weights: jnp.ndarray,
-    squashing_fun: Callable[[jnp.ndarray], jnp.ndarray] = None,
+    squashing_fun: Optional[Callable[[jnp.ndarray], jnp.ndarray]] = None,
     epsilon: float = 1e-2,
     **kwargs) -> jnp.ndarray:
   r"""Solves reg. OT, from inputs to a weighted family of increasing values.
@@ -41,6 +41,7 @@ def transport_for_sort(
     squashing_fun: function taking an array to squash all its entries in [0,1].
       sigmoid of whitened values by default. Can be set to be the identity by
       passing ``squashing_fun = lambda x : x`` instead.
+    epsilon: the regularization parameter.
     **kwargs: keyword arguments for `sinkhorn` and / or `PointCloud`.
 
   Returns:
@@ -54,15 +55,15 @@ def transport_for_sort(
 
   x = jnp.expand_dims(jnp.squeeze(inputs), axis=1)
   if squashing_fun is None:
-    squashing_fun = lambda z: jax.nn.sigmoid((z - jnp.mean(z)) /
-                                             (jnp.std(z) + 1e-10))
+    squashing_fun = lambda z: jax.nn.sigmoid(
+        (z - jnp.mean(z)) / (jnp.std(z) + 1e-10))
   x = squashing_fun(x)
   a = jnp.squeeze(weights)
   b = jnp.squeeze(target_weights)
   num_targets = b.shape[0]
   y = jnp.linspace(0.0, 1.0, num_targets)[:, jnp.newaxis]
 
-  return transport.Transport(x, y, a=a, b=b, epsilon=epsilon, **kwargs)
+  return transport.solve(x, y, a=a, b=b, epsilon=epsilon, **kwargs)
 
 
 def apply_on_axis(op, inputs, axis, *args, **kwargs):
@@ -75,6 +76,8 @@ def apply_on_axis(op, inputs, axis, *args, **kwargs):
       several axes are passed the operator, those are merged as a single
       dimension.
     *args: other positional arguments to the operator.
+    **kwargs: other positional arguments to the operator.
+
 
   Returns:
     A jnp.ndarray holding the output of the differentiable operator on the given
@@ -123,7 +126,7 @@ def _sort(inputs: jnp.ndarray, topk, num_targets, **kwargs) -> jnp.ndarray:
 def sort(inputs: jnp.ndarray,
          axis: int = -1,
          topk: int = -1,
-         num_targets: int = None,
+         num_targets: Optional[int] = None,
          **kwargs) -> jnp.ndarray:
   r"""Applies the soft sort operator on a given axis of the input.
 
@@ -167,7 +170,7 @@ def _ranks(inputs: jnp.ndarray, num_targets, **kwargs) -> jnp.ndarray:
 
 def ranks(inputs: jnp.ndarray,
           axis: int = -1,
-          num_targets: int = None,
+          num_targets: Optional[int] = None,
           **kwargs) -> jnp.ndarray:
   r"""Applies the soft trank operator on input tensor.
 
@@ -379,6 +382,7 @@ def quantize(inputs: jnp.ndarray,
   Args:
     inputs: the inputs as a jnp.ndarray[batch, dim].
     num_levels: number of levels available to quantize the signal.
+    axis: axis along which quantization is carried out.
     **kwargs: keyword arguments passed on to lower level functions. Of interest
       to the user are ``squashing_fun``, which will redistribute the values in
       ``inputs`` to lie in [0,1] (sigmoid of whitened values by default) to
